unit colorpicker;

interface
uses classes,controls,stdctrls,windows,graphics,extctrls,jpeg,sysutils,ComCtrls;

type TColorPicker=Class(TWinControl)
 private
 mainswatch : timage;
 sideswatch : timage;
 pal : Tbitmap;
 currenthue,currentsat,currentlum : integer;
 selecting,sideselecting : boolean;
 topoffset : integer;
 topmargin : integer;
 leftmargin : integer;
 fcolor : tcolor;
 FColChange : TNotifyEvent;
 ccanv : Tbitmap;
 curposx,curposy : integer;
 Procedure GenerateInitialColors;
 Procedure mainswatchmousedown(sender : TObject;Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
 Procedure mainswatchmouseup(sender : TObject;Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
 Procedure mainswatchmousemove(sender : TObject;Shift: TShiftState; X: Integer; Y: Integer);
 Procedure sideswatchmousedown(sender : TObject;Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
 Procedure sideswatchmouseup(sender : TObject;Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
 Procedure sideswatchmousemove(sender : TObject;Shift: TShiftState; X: Integer; Y: Integer);
 procedure drawsidetick(y : integer);
 procedure drawcursor;
 procedure setcolor(c : Tcolor);
protected
 Procedure Paint;
public
 Constructor Create(AOwner : TComponent); override;
 Destructor Destroy; override;
 Procedure Resize; override;
  property Col : TColor read fcolor write setcolor;
published
 property OnColorChange : TNotifyEvent read fColChange write fColChange;

End;

const
  RGBMAX = 255;
  HSLMAX=240;
  UNDEFINED = HSLMAX * 2 div 3;
  TICKWIDTH = 10;
var
test : array  [0..240,0..240] of integer;

procedure Register;
implementation



Constructor TColorPicker.Create(AOwner : TComponent);
begin
  inherited;
  //Create Components here
  Parent := (AOwner As TWinControl);
  mainswatch:=Timage.create(Self);
  mainswatch.Parent:=self;
  sideswatch:=Timage.create(Self);
  sideswatch.Parent:=self;
  pal:=Tbitmap.create;
  mainswatch.OnMouseDown:=mainswatchmousedown;
  mainswatch.OnMouseUp:=mainswatchmouseup;
  mainswatch.OnMouseMove:=mainswatchmousemove;
  sideswatch.OnMouseDown:=sideswatchmousedown;
  sideswatch.OnMouseUp:=sideswatchmouseup;
  sideswatch.OnMouseMove:=sideswatchmousemove;
  Caption:='';
  topoffset:=10;
  if (csDesigning in Componentstate) then begin
    //resize;
   // paint;
  end;
  ccanv:=tbitmap.create;

end;




function HueToRGB(n1,n2,hue : Integer) : word;
begin
  while hue > HSLmax do
    Dec (hue, HSLMAX);

  while hue < 0 do
    Inc (hue, HSLMAX);

  if hue < HSLMAX div 6 then
    result := ( n1 + (((n2-n1)*hue+(HSLMAX div 12)) div (HSLMAX div 6))
)
  else
    if hue < HSLMAX div 2 then
      result := n2
    else
      if hue < (HSLMAX*2) div 3 then
        result := n1 + (((n2-n1)*(((HSLMAX*2) div 3)-hue)+(HSLMAX div
12)) div (HSLMAX div 6))
      else
        result := n1
end;

function HSLToRGB (hue, lum, sat : Integer) : tcolor;
var
  Magic1,Magic2 : Integer;
  var r, g, b : Integer;

begin
  if sat = 0 then
  begin
    r := (lum * RGBMAX) div HSLMAX;
    g := r;
    b := r;
  end
  else
  begin
    if lum <= HSLMAX div 2 then
      Magic2 := (lum*(HSLMAX + sat) + (HSLMAX div 2)) div HSLMAX
    else
      Magic2 := lum + sat - ((lum*sat) + (HSLMAX div 2)) div HSLMAX;

    Magic1 := 2*lum-Magic2;

    r := (HueToRGB (Magic1,Magic2,hue+(HSLMAX div 3))*RGBMAX + (HSLMAX
div 2)) div HSLMAX;
    g := (HueToRGB (Magic1,Magic2,hue)*RGBMAX + (HSLMAX div 2)) div
HSLMAX;
    b := (HueToRGB (Magic1,Magic2,hue-(HSLMAX div 3))*RGBMAX + (HSLMAX
div 2)) div HSLMAX;
  end;

  result:=rgb(r,g,b);
end;



Procedure TColorPicker.GenerateInitialColors;
var
p : integer;
begin
pal.Canvas.Pen.Style:=psSolid;
pal.Canvas.Brush.Color:=color;
pal.Canvas.Pen.Color:=color;

pal.canvas.rectangle(0,0,pal.Width,pal.Height);
for p := 0 to HSLMAX do begin
 pal.Canvas.pen.color:=HSLTORGB(p,120,240);
 pal.Canvas.moveto(0,p);
 pal.Canvas.lineto(pal.width-TICKWIDTH,p);
end;
end;

Destructor TColorPicker.Destroy;
begin
  inherited;
  ccanv.free;
end;




Procedure TColorPicker.Resize;
begin
inherited;
topmargin:=height div 20;
leftmargin:=width div 20;
mainswatch.Left:=leftmargin;
mainswatch.Width:=round((width-(leftmargin*2)) / 1.2);
mainswatch.Height:=round((height-(topmargin*2)) / 1.1);
mainswatch.Top:=topmargin+topoffset;
ccanv.width:=mainswatch.width;
ccanv.height:=mainswatch.height;


sideswatch.Height:=mainswatch.Height+(topoffset*2);
sideswatch.Top:=topmargin;
sideswatch.width:= 20;
sideswatch.left:=mainswatch.left+mainswatch.Width+(width div 20);

pal.Width:=sideswatch.Width;
pal.height:=HSLMAX;
GenerateInitialColors;
paint;
drawsidetick(currenthue);
end;


Procedure TColorPicker.Paint;
var
x,xx,y,c : integer;
vcanv : tbitmap;
rout : pbytearray;
tr : Trect;
begin
inherited;
//if (csDesigning in Componentstate) then exit;
vcanv:=tbitmap.Create;
vcanv.Width:=HSLMAX;
vcanv.Height:=HSLMAX;

sideswatch.Canvas.brush.color:=color;
sideswatch.Canvas.brush.style:=bsSolid;
sideswatch.Canvas.Pen.style:=psSolid;
sideswatch.Canvas.Pen.color:=color;

sideswatch.Canvas.Rectangle(0,0,sideswatch.Width,sideswatch.height);
tr.Left:=0;
tr.Right:=sideswatch.width;
tr.Top:=topoffset;
tr.Bottom:=sideswatch.height-(topoffset);
sideswatch.Canvas.StretchDraw(tr,pal);


vcanv.PixelFormat:=pf24bit;
for y := 0 to vcanv.height-1 do begin
     rout:=vcanv.ScanLine[y];
     x:=0;
     xx:=0;
     while (x< 3*vcanv.Width-1) do begin
       c:=HSLtoRGB(currenthue,y,xx);
       rout[x]:=getbvalue(c);
       rout[x+1]:=getgvalue(c);
       rout[x+2]:=getrvalue(c);
       x:=x+3;
       inc(xx);
     end;

end;



//rgblabel.caption:=inttostr(test[20,20]);

//mainswatch.canvas.Draw(0,0,vcanv);
tr.left:=0;
tr.Right:=mainswatch.Width;
tr.Top:=0;
tr.Bottom:=mainswatch.Height;
mainswatch.Canvas.StretchDraw(tr,vcanv);
ccanv.Assign(mainswatch.Picture.bitmap);
drawcursor;
vcanv.Free;

end;


procedure Register;
begin
  RegisterComponents('LibrarySmith', [TColorPicker]);
end;

procedure TColorPicker.mainswatchmousedown(sender: TObject; Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
begin
selecting:=true;
mainswatchmousemove(sender,shift,X,Y);
end;

procedure TColorPicker.mainswatchmouseup(sender: TObject; Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
begin
mainswatchmousemove(sender,shift,X,Y);
selecting:=false;
end;


procedure TColorPicker.DrawCursor;
begin
 mainswatch.picture.Bitmap.assign(ccanv);
 mainswatch.Canvas.Pen.Width:=1;
 if (curposy<(mainswatch.height div 3)) then mainswatch.Canvas.Pen.Color:=clWhite else mainswatch.Canvas.Pen.Color:=clBlack;
 mainswatch.Canvas.MoveTo(curposx-1,curposy);
 mainswatch.Canvas.LineTo(curposx-7,curposy);
 mainswatch.Canvas.MoveTo(curposx+1,curposy);
 mainswatch.Canvas.LineTo(curposx+7,curposy);
 mainswatch.Canvas.MoveTo(curposx,curposy-1);
 mainswatch.Canvas.LineTo(curposx,curposy-7);
 mainswatch.Canvas.MoveTo(curposx,curposy+1);
 mainswatch.Canvas.LineTo(curposx,curposy+7);
end;

procedure TColorPicker.mainswatchmousemove(sender: TObject; Shift: TShiftState; X: Integer; Y: Integer);
begin


if selecting then begin
if x<0 then x:=0;
if x>mainswatch.width then x:=mainswatch.width;
if y<0 then y:=0;
if y>mainswatch.height then y:=mainswatch.Height;
 currentsat:=round(x*(HSLMAX / mainswatch.width));
 currentlum:=round(y*(HSLMAX / mainswatch.height));
 fcolor:=HSLTORGB(currenthue,currentlum,currentsat);
 if assigned(fcolchange) then FColChange(self);
 curposx:=x;
 curposy:=y;
 drawcursor;
end;


end;

procedure TColorPicker.sideswatchmousedown(sender: TObject; Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
begin
sideselecting:=true;
end;

procedure TColorPicker.sideswatchmouseup(sender: TObject; Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
begin
sideswatchmousemove(Sender,Shift,x,y);
sideselecting:=false;
end;


procedure TColorPicker.sideswatchmousemove(sender: TObject; Shift: TShiftState; X: Integer; Y: Integer);
begin
if sideselecting then begin
if (y<topoffset) then y:=topoffset;
if (y>sideswatch.height-topoffset) then y:=sideswatch.height-topoffset;
 currenthue:=round((y-topoffset)*(HSLMAX / (sideswatch.Height - (topoffset*2))));
 paint;
 fcolor:=HSLTORGB(currenthue,currentlum,currentsat);
 if assigned(fcolchange) then FColChange(self);
 drawsidetick(y-topoffset);
end;
end;

procedure tcolorpicker.drawsidetick(y: Integer);
begin
  y:=y+topoffset;
  sideswatch.Canvas.Brush.style:=bsSolid;
  sideswatch.Canvas.Pen.color:=clblack;
  sideswatch.Canvas.Brush.color:=clblack;
  sideswatch.Canvas.Pen.style:=psSolid;
  sideswatch.Canvas.moveto(sideswatch.Width-TICKWIDTH,y);
  sideswatch.Canvas.lineto(sideswatch.Width-1,y-4);
  sideswatch.Canvas.lineto(sideswatch.Width-1,y+4);
  sideswatch.Canvas.lineto(sideswatch.Width-TICKWIDTH,y);
  sideswatch.Canvas.FloodFill(sideswatch.Width-TICKWIDTH+2,y,clBlack,fsBorder);
end;

procedure RGBtoHSL(Red,Green,Blue : Integer;var Ho,So,Lo : integer);
var
Var_R,Var_G,Var_B : double;
var_Min,var_Max,del_Max : double;
H,L,S : double;
Del_R,Del_G,Del_B : double;

function max(v1,v2,v3 : double) : double;
begin
result:=0;
if (v1>=v2) and (v1>=v3)  then result:=v1;
if (v2>=v1) and (v2>=v3)  then result:=v2;
if (v3>=v2) and (v3>=v1)  then result:=v3;
end;

function min(v1,v2,v3 : double) : double;
begin
result:=0;
if (v1<=v2) and (v1<=v3)  then result:=v1;
if (v2<=v1) and (v2<=v3)  then result:=v2;
if (v3<=v2) and (v3<=v1)  then result:=v3;
end;



begin
h:=0;
Var_R := ( Red / 255 );                     //Where RGB values = 0  255
Var_G:= ( Green / 255 );
Var_B := ( Blue / 255 );

var_Min := min(Var_r,var_g,var_b);    //Min. value of RGB
var_Max := max(var_r,var_g,var_b);    //Max. value of RGB
del_Max := var_Max - var_Min;             //Delta RGB value

L := ( var_Max + var_Min ) / 2;

if ( del_Max = 0 ) then begin                   //This is a gray, no chroma...
   H := 0;                               //HSL results = 0  1
   S := 0;
end
else begin                                   //Chromatic data...

   if ( L < 0.5 ) then S := del_Max / ( var_Max + var_Min ) else S := del_Max / ( 2 - var_Max - var_Min );

   del_R := ( ( ( var_Max - var_R ) / 6 ) + ( del_Max / 2 ) ) / del_Max;
   del_G := ( ( ( var_Max - var_G ) / 6 ) + ( del_Max / 2 ) ) / del_Max;
   del_B := ( ( ( var_Max - var_B ) / 6 ) + ( del_Max / 2 ) ) / del_Max;

   if ( var_R = var_Max ) then H := del_B - del_G
   else if ( var_G = var_Max ) then H := ( 1 / 3 ) + del_R - del_B
   else if ( var_B = var_Max ) then H:= ( 2 / 3 ) + del_G - del_R;

   if ( H < 0 ) then  H:=H+1;
   if ( H > 1 ) then  H:=H-1;
end;

Ho:=round(HSLMAX*H);
So:=round(HSLMAX*S);
Lo:=round(HSLMAX*L);

end;



procedure tcolorpicker.setcolor(c: TColor);
var
r,g,b : byte;
hh,ss,ll : integer;
begin
r:=getrvalue(c);
g:=getgvalue(c);
b:=getbvalue(c);


RGBToHSL(r,g,b,hh,ss,ll);
currenthue:=hh;
currentsat:=ss;
currentlum:=ll;
drawsidetick(currenthue);
paint;
curposx:=ss;
curposy:=ll;
drawcursor;

fcolor:=c;
OnColorChange(self);

{sideselecting:=true;
selecting:=true;
sideswatchmousemove(nil,[],4,hh+topoffset);
mainswatchmousemove(nil,[],ss,ll);
selecting:=false;
sideselecting:=false;
}
end;

end.
