unit radioscroller;

interface
uses windows,graphics,controls,classes,extctrls,sysutils,jpeg;


const MORECOLOUR = clwhite;

type tradio = record
pic : tbitmap;
title : string;
description : string;
url : string;
end;

type tradiolist = record
count : integer;
lst : array of tradio;
end;

type tmatrix = record
x1,y1,x2,y2 : integer;
item : integer;
end;



type
  TRadioScroller = class(TGraphicControl)
  private
  matrix : array of tmatrix;
  ftopmost : integer;
  fcolor : Tcolor;
  facross : integer;
  pl : tradiolist;
  fcount : integer;
  fitemindex : integer;
  repeattimer : ttimer;    { Private declarations }
  xx,yy : integer;
  isdown : boolean;
  marked : integer;
  fremote : boolean;
  remotex,remotey : integer;
  visrows,viscols : integer;
  remoteselection : tmatrix;
  fscalescrolls : real;
  fscrollwidths : integer;

    procedure setcolor(_color : tcolor);
    procedure setacross(acr : integer);
    procedure repeattimertimer(sender : tobject);
    function getpic : tbitmap;
    function finditem (x : integer) : integer;

    procedure scaleicon(var bmp : tbitmap);


  protected
    { Protected declarations }
  public
  constructor Create(AOwner : TComponent); override;
  destructor Destroy; override;
  procedure Paint; override;
  procedure Resize; override;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState;X, Y: Integer);override;
  procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState;X, Y: Integer);override;
  procedure add(bmp : tbitmap;title : string; description : string;url : string);
  procedure delete(i : integer);
  procedure replace(i : integer;bmp : tbitmap);
  procedure clear;
  procedure Click; override;
  procedure Makeitemvisible(i : integer);
    { Public declarations }
  property Picture : tbitmap read getpic;
  procedure Moveup;
  procedure Movedown;
  procedure Moveleft;
  procedure Moveright;
  procedure save(fn : string);
  function load(fn : string) : boolean;

  published
     { Published declarations }
   property items : tradiolist read pl;
   property color : Tcolor read Fcolor write setcolor default clred;
   property across : integer read Facross write setacross default 4;
   property itemindex : integer read fitemindex;
   property OnClick;
   property topmost : integer read ftopmost;
   property remote : boolean read fremote write fremote;
   property scrollwidths : integer read fscrollwidths write fscrollwidths;
   procedure remoteclick;
   property count : integer read fcount;
  end;


const
TIMERINTERVAL=250;



procedure Register;

implementation
{$R ICONS.RES}

function wraptext(left,top,width,bottom : integer;text : string;canv : tcanvas) : integer;
type tlines = record
 line : array of string;
 count  : integer;
end;

var
lines : tlines;
op : tlines;


function getlastspecialchar(text : string) : integer;
var
chars : string;
x,n : integer;
begin
chars:=#39' ,.|\/@:~#{}][`!"$%^&*()_+=';

result:=0;
for x:=length(text) downto 1 do begin

    for n:=1 to length(chars) do begin
    if text[x]=chars[n] then begin
    result:=x;
    exit;
    end;
    end;

end;
end;



var
n : integer;
l : integer;
s : string;
atxt : string;
begin
result:=top;
atxt:=text;

if canv.textwidth(text) <= width then begin
canv.TextOut(left,top,text);
result:=top;
result:=result+canv.textheight(text);
exit;
end;

lines.count:=0;
l:=getlastspecialchar(text);
while(l<>0) do begin
inc(lines.count);
setlength(lines.line,lines.count);
lines.line[lines.count-1]:=copy(text,l,length(text));
delete(text,l,length(text));
l:=getlastspecialchar(text);
end;

inc(lines.count);
setlength(lines.line,lines.count);
lines.line[lines.count-1]:=text;


s:='';
l:=lines.count-1;
op.count:=0;
while(l>-1) do begin
if canv.textwidth(lines.line[l])>width then begin
canv.textout(left,top,copy(atxt,1,25)+'...');
result:=top+canv.textheight(text);
exit;
end;

 if canv.textwidth(s+lines.line[l])<width then s:=s+lines.line[l] else begin
    inc(op.count);
    setlength(op.line,op.count);
    op.line[op.count-1]:=s;
    inc(l);
    s:='';
 end;
 dec(l);
 end;




inc(op.count);
setlength(op.line,op.count);
op.line[op.count-1]:=s;



for n:=0 to op.count-1 do begin
if (op.line[n]<>'') and (top+(canv.textheight(op.line[n])*(n+1))<bottom) then canv.textout(left,top+(canv.textheight(op.line[n])*n),trim(op.line[n]));
result:=top+(canv.textheight(op.line[n])*n);
end;
result:=result+canv.textheight(text);


end;




procedure TRadioScroller.remoteclick;
begin
MouseDown(mbLeft,[],remoteselection.x1+1,remoteselection.y1+1);

end;

function TRadioScroller.finditem(x : integer) : integer;
var
n : integer;
begin
result:=-99;
for n:=0 to fcount+2 do begin
    if x=matrix[n].item then begin
    result:=n;
    break;
    end;
end;
end;



procedure TRadioScroller.Moveup;
var
i : integer;
begin
if remotey>0 then dec(remotey) else begin
   i:=finditem(-2);
   if i<>-99 then begin
      MouseDown(mbLeft,[],matrix[i].x1+1,matrix[i].y1+1);
   end;
end;
repaint;
end;


procedure TRadioScroller.Movedown;
var
i : integer;
begin
if remotey<visrows then inc(remotey) else begin
   i:=finditem(-1);
   if i<>-99 then begin
      MouseDown(mbLeft,[],matrix[i].x1+1,matrix[i].y1+1);
   end else inc(visrows);
end;
if topmost+((remotey*facross)+remotex-1)>fcount-2 then begin;
dec(remotey);
end;
repaint;
end;

procedure TRadioScroller.Moveleft;
begin
if remotex>0 then dec(remotex);
repaint;
end;

procedure TRadioScroller.Moveright;
begin
if remotex<viscols-1 then inc(remotex);
if topmost+((remotey*facross)+remotex-1)>fcount-2 then begin;
dec(remotex);
end;
repaint;
end;


{procedure TRadioScroller.createparams(Var params : TCreateParams);
begin
inherited createparams(params);
params.ExStyle:=params.ExStyle or WS_EX_TRANSPARENT;
end;

{procedure TRadioScroller.WMEraseBkGnd(var msg : TWMEraseBkGnd);
begin
SetBkMode(msg.DC,TRANSPARENT);
msg.result:=1;
end;
}

procedure TRadioScroller.Makeitemvisible(i : integer);

begin
{if i>fcount-1 then exit;
x:=0;
y:=0;
for n:=0 to i do begin
if n=i then break;
inc(x);
if x=facross then begin; x:=0;inc(y);end;
end;                                }

ftopmost:=i div across * across;


paint;

//fitemindex:=-1;
//mouseup(mbleft,[],0,0);



end;

procedure TRadioScroller.replace(i : integer;bmp : tbitmap);
var
picturewidth : integer;
tr : trect;
begin
if (i<0) or (i>fcount) then exit;
 picturewidth:=width div facross;
  tr.left:=0;
  tr.top:=0;
  tr.bottom:=picturewidth;
  tr.right:=picturewidth;
  tr.bottom:=picturewidth div 2;
  pl.lst[i].pic.canvas.stretchdraw(tr,bmp);
  pl.lst[i].pic.canvas.Brush.color:=clblack;
  pl.lst[i].pic.canvas.Brush.Style:=bsSolid;
  tr.top:=picturewidth div 2;
  tr.Bottom:=picturewidth;
  pl.lst[i].pic.canvas.fillrect(tr);
end;


procedure TRadioScroller.delete(i : integer);
var
n : integer;
begin
for n:=i to fcount-2 do begin
    pl.lst[n].pic.assign(pl.lst[n+1].pic);
end;
dec(fcount);
setlength(pl.lst,fcount);
repaint;
end;


procedure TRadioScroller.add(bmp : tbitmap;title : string;description : string;url : string);
var
picturewidth : integer;
spacing : integer;
tr : trect;
begin
picturewidth:=width div facross;
spacing:=picturewidth div 10;
picturewidth:=picturewidth - spacing;

setlength(pl.lst,fcount+1);
pl.lst[fcount].pic:=tbitmap.create;
pl.lst[fcount].pic.width:=picturewidth;
pl.lst[fcount].pic.height:=picturewidth div 2;

//pl.pic[fcount].assign(bmp);



  tr.left:=0;
  tr.top:=0;
  tr.bottom:=picturewidth div 2;
  tr.right:=picturewidth;
  pl.lst[fcount].pic.canvas.stretchdraw(tr,bmp);
  pl.lst[fcount].title:=title;
  pl.lst[fcount].description:=description;
  pl.lst[fcount].url:=url;
inc(fcount);
//paint;
end;

procedure TRadioScroller.clear;
var
n : integer;
begin
if fcount=0 then exit;
for n:=0 to fcount do pl.lst[n].pic.free;
fcount:=0;
setlength(pl.lst,fcount);
end;

function TRadioScroller.getpic : tbitmap;
begin
result:=tbitmap.create;
if assigned(pl.lst[fitemindex].pic) then result.assign(pl.lst[fitemindex].pic);
end;

procedure TRadioScroller.setcolor(_color : tcolor);
begin
Fcolor:=_color;
end;

procedure TRadioScroller.setacross(acr : integer);
begin
Facross:=acr;
end;

constructor TRadioScroller.Create(AOwner : TComponent);
begin
     inherited create(aowner);
     Parent := (AOwner AS TWinControl);
     height:=200;
     width:=200;


     facross:=4;
     ftopmost:=0;
     repeattimer:=ttimer.create(self);
     repeattimer.interval:=TIMERINTERVAL;
     repeattimer.ontimer:=repeattimertimer;
     repeattimer.enabled:=false;
     controlstyle:=controlstyle - [csOpaque];
     scrollwidths:=18;

end;

destructor TRadioScroller.Destroy;
var
n : integer;
begin
for n:=0 to fcount-1 do pl.lst[n].pic.free;
inherited destroy;
end;

procedure TRadioScroller.scaleicon(var bmp : tbitmap);
var
obmp : tbitmap;
tr : trect;
begin
obmp:=tbitmap.create;
fscalescrolls:=fscrollwidths / bmp.width;
obmp.width:=round(bmp.width*fscalescrolls);
obmp.height:=round(bmp.height*fscalescrolls);

tr.top:=0;
tr.left:=0;
tr.Right:=obmp.width;
tr.Bottom:=obmp.height;
obmp.Canvas.StretchDraw(tr,bmp);
bmp.assign(obmp);
obmp.free;
end;


procedure TRadioScroller.Paint;
const
scrollerheight = 18;
var


n,z : integer;
picturewidth,spacing : integer;
x,y : integer;
tr,dr,txr : trect;
o : tbitmap;
ht : integer;
piccount : integer;
hoffset : integer;
vcanv : tbitmap;

begin
//inherited;
vcanv:= Tbitmap.create;
vcanv.width:=width;
vcanv.height:=height;
vcanv.canvas.brush:=Canvas.brush;
vcanv.canvas.pen:=Canvas.pen;
vcanv.canvas.font:=canvas.font;

tr.top:=0;
tr.left:=0;
tr.right:=width;
tr.bottom:=height;
vcanv.canvas.copyrect(tr,canvas,tr);
viscols:=facross;
visrows:=0;


with vcanv do begin

canvas.brush.style:=bsclear;
o:=tbitmap.create;

if (facross=0) or (fcount=0) then begin
   vcanv.free;
   o.free;
   exit;
end;

//canvas.brush.color:=fcolor;
//canvas.Rectangle(0,0,width,height);



setlength(matrix,fcount+3);


picturewidth:=width div facross;



spacing:=picturewidth div 10;

ht:=(((height-scrollerheight*2) div picturewidth)*(picturewidth))+(scrollerheight*2);
hoffset:=(height div 2) - (ht div 2)-spacing;
ht:=ht+hoffset;

picturewidth:=picturewidth - spacing;


if ftopmost<>0 then begin
     if marked=-2 then o.LoadFromResourceName(hinstance,'PUPPRESSED') else o.LoadFromResourceName(hinstance,'PUPNORMAL');
      scaleicon(o);
///o.LoadFromResourceName(hinstance,'PUPNORMAL');

tr.top:=hoffset;
tr.left:=(width div 2) - (o.width div 2);
tr.right:=tr.left+o.width;
tr.Bottom:=o.height+hoffset;

dr.Left:=0;
dr.top:=0;
dr.right:=o.width;
dr.Bottom:=o.height;
canvas.brush.style:=bsclear;

canvas.BrushCopy(tr,o,dr,RGB(255,255,255));
canvas.font.size:=round(7*fscalescrolls);
canvas.font.name:='Arial';
canvas.font.color:=MORECOLOUR;
canvas.textout(width div 2 - canvas.textwidth('More') div 2,(tr.bottom)+1,'More');
canvas.Brush.style:=bsClear;

matrix[0].x1:=tr.left;
matrix[0].x2:=tr.right;
matrix[0].y1:=tr.top;
matrix[0].y2:=tr.bottom;
matrix[0].item:=-2;

end else matrix[0].item:=-3;






y:=hoffset+scrollerheight+spacing;

x:=spacing div 2;
//y:=(scrollerheight*3) div 2;

piccount:=0;
for n:=1 to fcount-(ftopmost) do begin
tr.top:=y;
tr.bottom:=y+(picturewidth);
tr.left:=x;
tr.right:=x+picturewidth;
matrix[n].x1:=tr.left;
matrix[n].x2:=tr.right;
matrix[n].y1:=tr.top;
matrix[n].y2:=tr.bottom;
matrix[n].item:=ftopmost+(n-1);
canvas.brush.color:=clBlack;
canvas.brush.style:=bsSolid;
canvas.rectangle(matrix[n].x1,matrix[n].y1,matrix[n].x2,matrix[n].y2);
txr.bottom:=tr.bottom - (picturewidth div 2)-5;
txr.top:=tr.top+5;
txr.left:=tr.left+5;
txr.right:=tr.right-5;
canvas.stretchdraw(txr,pl.lst[ftopmost+(n-1)].pic);
canvas.Pen.color:=clAqua;
canvas.brush.style:=bsClear;
canvas.Rectangle(txr.left,txr.top,txr.right,txr.bottom);
canvas.Font.color:=clWhite;
canvas.Font.name:='Arial';
canvas.Font.size:=8;
canvas.font.style:=[fsBold];
canvas.TextOut(tr.left+5,tr.bottom- (picturewidth div 2),pl.lst[ftopmost+(n-1)].title);
canvas.font.style:=[];
txr.Left:=tr.left+5;
txr.Right:=picturewidth-5;
txr.Top:=tr.bottom - (picturewidth div 2)+canvas.textheight('l');
txr.bottom:=tr.bottom;
wraptext(txr.left,txr.top,txr.right,txr.bottom,pl.lst[ftopmost+(n-1)].description,canvas);


if fremote=true then begin
canvas.pen.width:=10;
canvas.pen.color:=clred;
if n=((remotey*facross)+remotex) then begin
 canvas.Rectangle(tr.left,tr.top,tr.Right,tr.Bottom);
 remoteselection:=matrix[n+1];
end;
end;

inc(piccount);
x:=x+picturewidth+spacing;
if x>width then begin
y:=y+picturewidth+(spacing);
x:=spacing div 2;
if y+picturewidth>ht then begin
   break;
end;

inc(visrows);
end;


end;


if ((ftopmost+(piccount-2)) = fcount-2) then begin
 matrix[n+1].item:=-3
  end

 else begin
o.free;
o:=tbitmap.create;
//o.LoadFromResourceName(hinstance,'PDOWNNORMAL');
   if marked=-1 then o.LoadFromResourceName(hinstance,'PDOWNPRESSED') else o.LoadFromResourceName(hinstance,'PDOWNNORMAL');
      scaleicon(o);
tr.top:=y+spacing;
tr.left:=(width div 2) - (o.width div 2);
tr.right:=tr.left+o.width;
tr.Bottom:=tr.top+o.height;

dr.Left:=0;
dr.top:=0;
dr.right:=o.width;
dr.Bottom:=o.height;
canvas.BrushCopy(tr,o,dr,RGB(255,255,255));
canvas.font.size:=round(7*fscalescrolls);
canvas.font.name:='Arial';
canvas.font.color:=MORECOLOUR;
canvas.textout(width div 2 - canvas.textwidth('More') div 2,tr.top-canvas.textheight('More'),'More');

matrix[n+1].x1:=tr.left;
matrix[n+1].x2:=tr.right;
matrix[n+1].y1:=tr.top;
matrix[n+1].y2:=tr.bottom;
matrix[n+1].item:=-1;
end;

z:=n;

for n:=z+1 to fcount do begin
matrix[n+1].item:=-3;
end;

end;

canvas.draw(0,0,vcanv);
vcanv.free;

o.free;
end;

procedure TRadioScroller.Resize;
begin
paint;
end;

procedure TRadioScroller.MouseUp(Button: TMouseButton; Shift: TShiftState;X, Y: Integer);
var
n : integer;
begin
marked:=-3;
isdown:=false;
repeattimer.enabled:=false;
repeattimer.interval:=TIMERINTERVAL;
//fitemindex:=-1;
//inherited MouseDown(Button,Shift,X,Y);
if fcount=0 then begin
   inherited click;
   exit;
end;

for n:=0 to fcount+2 do begin
    if (x>=matrix[n].x1) And
       (x<=matrix[n].x2) And
       (y>=matrix[n].y1) And
       (y<=matrix[n].y2) then begin
                               //marked:=matrix[n].item;
                               fitemindex:=matrix[n].item;
                               //if matrix[n].item>-1 then if fselected[matrix[n].item]=false then fselected[matrix[n].item]:=true else fselected[matrix[n].item]:=false;
                               if matrix[n].item>-1 then inherited click;
                               repaint;
                               exit;
                              end;

end;


end;

procedure TRadioScroller.MouseDown(Button: TMouseButton; Shift: TShiftState;X, Y: Integer);
var
n : integer;
begin
isdown:=true;
repeattimer.enabled:=true;

//fitemindex:=-1;
//inherited MouseDown(Button,Shift,X,Y);
if fcount=0 then begin
   inherited click;
   exit;
end;
for n:=0 to fcount+2 do begin
    if (x>=matrix[n].x1) And
       (x<=matrix[n].x2) And
       (y>=matrix[n].y1) And
       (y<=matrix[n].y2) then begin
                               marked:=matrix[n].item;
                               fitemindex:=matrix[n].item;
                               //if matrix[n].item>-1 then if fselected[matrix[n].item]=false then fselected[matrix[n].item]:=true else fselected[matrix[n].item]:=false;
                               if matrix[n].item=-1 then ftopmost:=ftopmost+across;
                               if matrix[n].item=-2 then ftopmost:=ftopmost-across;
                               repaint;
                               exit;
                              end;

end;
end;


procedure TRadioScroller.click;
begin
end;

procedure Register;
begin
  RegisterComponents('Samples', [TRadioScroller]);
end;

procedure TRadioScroller.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
xx:=x;
yy:=y;
//if isdown=true then mouseup(tmousebutton(0),shift,x,y);
end;

procedure TRadioScroller.repeattimertimer(sender : tobject);
begin
mousedown(tmousebutton(0),[],xx,yy);
//sendmessage(self.handle,WM_LBUTTONDOWN,xx,yy);
if repeattimer.interval>20 then repeattimer.interval:=repeattimer.interval-10;

end;


procedure TRadioScroller.save(fn : string);
var
fs : tfilestream;
n : integer;
jp : tjpegimage;
ms : Tmemorystream;
sbuf : string;

function formatlength(l : integer) : string;
var
sh : string;
begin
sh:=format('%.8d',[l]);
result:=sh;
end;

procedure writestring(st : string);
var
ln : integer;
begin
    ln:=length(st);
    sbuf:=formatlength(ln);
    fs.Write(sbuf[1],length(sbuf));
    fs.Write(st[1],ln);
end;


begin
fs:=tfilestream.create(fn,fmCreate);

n:=0;
while(n<fcount) do begin
    jp:=tjpegimage.create;
    jp.Assign(pl.lst[n].pic);
    jp.jpegneeded;
    ms:=Tmemorystream.Create;
    jp.SaveToStream(ms);
    jp.free;
    sbuf:=formatlength(ms.size);
    fs.Write(sbuf[1],length(sbuf));
    ms.SaveToStream(fs);
    ms.free;
    writestring(pl.lst[n].title);
    writestring(pl.lst[n].description);
    writestring(pl.lst[n].url);
    inc(n);
    end;
fs.free;

end;

function TradioScroller.load(fn : string) : boolean;
var
fs : tfilestream;
ms : tmemorystream;
jp : tjpegimage;
done : boolean;
error : boolean;
ln : integer;
title,description,url : string;
bp : tbitmap;

function getlength : integer;
var
v,c : integer;
ss : string;
s : string;
begin
setlength(s,8);
fs.Read(s[1],8);
s:=trim(s);
val(s,v,c);
result:=v;
if c<>0 then result:=-1;
end;

function readstring(var s : String) : boolean;
var
ln : integer;
ss : string;
begin
result:=true;
ln:=getlength;
if ln=-1 then begin
   result:=false;
   exit;
end;
setlength(ss,ln);
fs.Read(ss[1],ln);
s:=ss;
end;


begin
clear;
result:=true;
error:=false;
done:=false;
fs:=tfilestream.Create(fn,fmOpenRead);

while done=false do begin

ln:=getlength;
if ln=-1 then begin; fs.free; exit;end;

ms:=tmemorystream.Create;
ms.CopyFrom(fs,ln);
jp:=tjpegimage.Create;
ms.Seek(0,soFromBeginning);
jp.PixelFormat:=jf24bit;
jp.LoadFromStream(ms);
ms.free;


if readstring(title)=false then begin; fs.free; exit;end;

if readstring(description)=false then begin; fs.free; exit;end;

if readstring(url)=false then begin; fs.free; exit;end;

bp:=tbitmap.create;
bp.Assign(jp);

add(bp,title,description,url);

bp.free;


if fs.Position=fs.size then done:=true;

end;

fs.Free;

result:=true;
paint;


end;

end.
